## Introduction

[RDMS](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms) is an application based on [Hyrax 3.3](https://github.com/samvera/hyrax/releases/tag/v3.3.0) stack by [Cottage Labs](https://cottagelabs.com/) and [AntLeaf](https://antleaf.com/). It is built with Docker containers.

## Getting Started

Clone the repository with `git clone https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms.git`.

Ensure you have docker and docker-compose.

Open a console and try running `docker -h` and `docker-compose -h` to verify they are both accessible.

Create the environment file `.env`. You can start by copying the template file [.env.template.development](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/.env.template.development) to `.env` and customizing the values to your setup. <br>
Note: For production environment, use .env.template as your template.

## Quick start
If you would like to do a test run of the system, start the docker containers
```bash
$ cd rdms
$ docker-compose -f docker-compose.yml -f docker-compose.development.override.yml up -d
```
You should see the containers being built and the services start.

## Docker compose explained

There are 2 `docker-compose` files provided in the repository, which build the containers running the services as shown above
* [docker-compose.yml](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml) is the main docker-compose file. It builds all the core servcies required to run the application

* [docker-compose.development.override.yml](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/5911149d9efa48ddbb19b82294dcbc74bcf1fb2b/docker-compose.development.override.yml) is used along with the main [docker-compose.yml](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml) file in development, mainly to expose ports for the various services and mount the local development environment for rdms.

### Containers running in docker

* [fcrepo](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L13-L22) is the container running the [Fedora 4 commons repository](https://wiki.duraspace.org/display/FEDORA47/Fedora+4.7+Documentation), an rdf document store. 

  By default, this runs the fedora service on port 8080 internally in docker.
  http://fcrepo:8080/fcrepo/rest

* [Solr container](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L24-L45) runs [SOLR](lucene.apache.org/solr/), an enterprise search server. 

  By default, this runs the SOLR service on port 8983 internally in docker. 
  http://solr:8983

* [db containers](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L47-L76) running a postgres database for use by the Hyrax application (appdb) and Fedora (fcrepodb). 

  By default, this runs the database service on port 5432 internally in docker.

* [redis container](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L121-L132) running [redis](https://redis.io/), used by Hyrax to manage background tasks. 

  By default, this runs the redis service on port 6379 internally in docker.

* [app container](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L78-L96) sets up the Hyrax application, which is then used by 2 services - web and workers.

  * [Web container](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L98-L110) runs the application. 

    By default, this runs on port 3000 internally in docker.
    http://web:3000 

*  [Wokers container](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/docker-compose.yml#L112-L119) runs the background tasks, using [sidekiq](https://github.com/mperham/sidekiq) and redis. 

  * Hyrax processes long-running or particularly slow work in background jobs to speed up the web request/response cycle. When a user submits a file through a work (using the web or an import task), there a number of background jobs that are run, initilated by the hyrax actor stack, as explained [here](https://samvera.github.io/what-happens-deposit-2.0.html).

    You can monitor the background workers using the materials data repository service at http://web:3000/sidekiq when logged in as an admin user. 
### Setup of RDMS at startup

The RDMS web container is the main entry point of the rdms application, with which users interact.

At startup, the web container runs [docker-entrypoint.sh](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/rdms/-/blob/master/hyrax/docker-entrypoint.sh). This script does the following tasks 

#### 1. Initial checks

* Creates the log folder if it doesn't exist
* Checks the bundle (and installs It in development)
* Does the database migration and setup
* Checks Solr and Fedora are running (waits 15 seconds if needed)

#### 2. RDMS creates some default users at start-up

* System users if they don't exist

  * Sytem administrator - the email id of the user is defined in the env file as `SYSTEM_ADMINISTRATOR`
  * Publication manager -  the email id of the user is defined in the env file as `SYSTEM_PUBLICATION_MANAGER`

* Creates users defined in the file `hyrax/seed/setup.json`, if they haven't already been created.

  **Setting a password for the system users**

  The system users are created with a random password. If you need to login as these users, you need to change the password from the rails console (once the web container is up and running)

  ```
  docker exec -it rdms-web-1 /bin/bash
  rails c
  u = User.find_by(email: ENV['SYSTEM_ADMINISTRATOR'])
  u.password = <some password>
  u.save
  ```

  **Creating a set of users during start-up from `setup.json`**

  RDMS uses the file `hyrax/seed/setup.json` to create a set of users during first startup, if the file exists. 

  If you would like to create users during startup, 

  * The file `hyrax/seed/setup.json` needs to exist at build time.

  * Copy the file in `hyrax/seed/setup.json.template` to `hyrax/seed/setup.json`

    * Modify `hyrax/seed/setup.json` so it has the list of users to create / update.

      For more information on the rake task to create users and the json file, see [this wiki](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/antleaf-projectmanagement/-/wikis/Development-notes/Create-users-from-the-rails-console) page. 

  * Build the image

  Note

  If the container is re-started, 

  * The task will go through the list of users in `hyrax/seed/setup.json` and check they exists. Their details will not be updated, if setup.json is modified.

  * If the file is deleted, it will skip checking the users exist or creating them if they exist.

  * You do not need to have the file `hyrax/seed/setup.json` at build time to create users at start-up. 

    * The rake task  [setup_users.rake](https://gitlab.ruhr-uni-bochum.de/FDM/rdm-system/antleaf-projectmanagement/-/wikis/Development-notes/Create-users-from-the-rails-console) can be used to create users as explained in the wiki page. Once the web container is up, 

      * Execute a shell on the container
      * Prepare a json file with the list of users
      * run the rake task

      This is better than having a list of users at build time, as it only needs to be run once.

#### 3. Loads workflows, create default admin sets and collection types

* Loads the default workflows 

* Creates the default admin set and collection types

#### 4. Starts the rails server


### Container volumes
The data for the application is stored in docker named volumes as specified by the compose files. These are:

```bash
$ docker volume list -f name=rdms
DRIVER    VOLUME NAME
local     rdms_app
local     rdms_cache
local     rdms_db-app
local     rdms_db-fcrepo
local     rdms_derivatives
local     rdms_fcrepo
local     rdms_file_uploads
local     rdms_redis
local     rdms_solr
```

These will persist when the system is brought down and rebuilt. Deleting them will require importers etc. to run again.


## Running in development or test

When running in development and test environment, prepare your .env file using .env.template.development as the template. You need to use `docker-compose -f docker-compose.yml -f docker-compose.development.override.yml`. This will use the docker-compose.yml file and the docker-compose.development.override.yml file.
  * fcrepo container will run the fedora service, which will be available in port 8080 at  http://localhost:8080/fcrepo/rest
  * Solr container will run the Solr service, which will be available in port 8983 at  http://localhost:8983
  * The web container runs the materials data repository service, which will be available in port 3000 at http://localhost:3000

You could setup an alias for docker-compose on your local machine, to ease typing

```bash
alias hd='docker-compose -f docker-compose.yml -f docker-compose.development.override.yml'
```

### Yarn and static assets

Static asset build is only run in `production` environment to speed up container creation in develop. To see features such as the IIIF viewer, `yarn install` must be run on the web container once it's up.

```
hd run web yarn install
```

For production envionment the assets precompile may fail. If rdms comes up with an error page this might have happed. Precompile the assets within the web container:

get into the web container 
```
docker exec -it rdms-web-1 /bin/bash
```

From within the web container precompile assets 
```
bundle exec rake assets:clean assets:precompile
```

## Builidng, starting and managing the service with docker

### Build the docker container

To start with, you would need to build the system, before running the services. To do this you need to issue the `build` command
```bash
$ hd build
```

### Start and run the services in the containers

To run the containers after build, issue the `up` command (-d means run as daemon, in the background):

```bash
$ hd up -d
```
The containers should all start and the services should be available in their end points as described above
* web server at http://localhost:3000 in development and https://domain-name in production

### docker container status and logs

You can see the state of the containers with `docker-compose ps`, and view logs e.g. for the web container using `docker-compose logs web`

The services that you would need to monitor the logs for are docker mainly web and workers.


### Some example docker commands and usage:

[Docker cheat sheet](https://github.com/wsargent/docker-cheat-sheet)

```bash
# Bring the whole application up to run in the background, building the containers
hd up -d --build

# Stop the container
hd stop

# Halt the system
hd down

# Re-create the web container without affecting the rest of the system (and run in the background with -d)
hd up -d --build --no-deps --force-recreate web

# View the logs for the web application container
hd logs web

# Create a log dump file
hd logs web | tee web_logs_`date --iso-8601`
# (writes to e.g. web_logs_2022-02-14)

# View all running containers
hd ps     

# Using its container name, you can run a shell in a container to view or make changes directly
docker exec -it rdms-web-1 /bin/bash
```

## Backups

There is [docker documentation](https://docs.docker.com/storage/volumes/#backup-restore-or-migrate-data-volumes) advising how to back up volumes and their data.



