
# frozen_string_literal: true
namespace :hyrax do
  namespace :default_admin_set_for_workflow do
    desc "Create the Default Admin Set"
    task create: :environment do
      admin_role = Role.find_by(name: "admin")
      admin_user = admin_role&.users&.first

      if admin_user.present?
        rub_default_admin_set_title = ENV.fetch('RUB_ADMIN_SET_TITLE', 'RUB publication workflow')
        crc_default_admin_set_title = ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow')

        [rub_default_admin_set_title, crc_default_admin_set_title].each do |title|
          unless AdminSet.where(title: [title]).present?
            admin_set = AdminSet.new(title: [title])
            admin_set.save

            Hyrax::AdminSetCreateService.call(admin_set: admin_set, creating_user: admin_user)

            workflow_name = title == rub_default_admin_set_title ? 'rub_publication_workflow' : 'crc_1280_publication_workflow'

            mediated_workflow = admin_set.permission_template.available_workflows.where(name: workflow_name).first

            Sipity::Workflow.activate!(permission_template: admin_set.permission_template, workflow_id: mediated_workflow.id)

            # Update Participants For RUB AdminSet
            if title == rub_default_admin_set_title
              access_grants_params_for_all_users = {access_grants_attributes:{'0'=> {agent_type: "group", agent_id: 'registered', access:"deposit"}}}
              permission_template_form =  Hyrax::Forms::PermissionTemplateForm.new(admin_set.permission_template)
              permission_template_form.update(access_grants_params_for_all_users)
            end
          end
        end
      else
        puts "Admin Sets not created, No Admin user exist"
      end
    end
  end
end
